using System.Collections.ObjectModel;
using System.Linq;

using VirtualLabAPI.Core.Common;
using VirtualLabAPI.Core.LightPath;
using VirtualLabAPI.UI.LightPath;

namespace OwnCode {
	/// <summary>
	/// Module for generating your own customized Tree of Optical Setup Elements. To work properly, it must be derived from 
	/// IOpticalSetupTreeGenerator.
	/// When you save this module somewhere within "%APPDATA%\Wyrowski Photonics GmbH\VirtualLab Fusion\{current version}" and
	/// run this module normally, all open Optical Setups are updated to use this tree.
	/// </summary>
	/// <remarks>Version 1.0c</remarks>
	public class VLModule : IOpticalSetupTreeGenerator {

		/// <summary>
		/// Method to generate the new Optical Setup Tree.
		/// </summary>
		/// <returns>The customized Optical Setup Tree.</returns>
		public OpticalSetupTree GenerateOpticalSetupTree() {

			Collection<OpticalSetupTreeElement> lensAnalysisCollection = new();
			#region Light Sources
			lensAnalysisCollection.Add("Light Sources|Plane Wave", OpticalSetupTreeElement.PlaneWave());
			lensAnalysisCollection.Add("Light Sources|Spherical Wave (Small NA)", OpticalSetupTreeElement.SphericalWave());
			lensAnalysisCollection.Add("Light Sources|Spherical Wave (Medium NA)", new OpticalSetupTreeElement(ElementType.LightSource, typeof(SphericalWaveLightSourceLPE),
				(opticalSetupType) => {
					SphericalWaveLightSourceLPE mySpherical = new SphericalWaveLightSourceLPE();
					mySpherical.LightSourceOPS.BasicParameter.DistanceToOrigin = 5e-3;
					return mySpherical;

				}, LightPathType.General, LightPathType.NearEyeDisplay));
			#endregion

			#region components
			lensAnalysisCollection.Add("Components|Component from Catalog", OpticalSetupTreeElement.LoadFromComponentCatalog());
			lensAnalysisCollection.Add("Components|Plane Surface (k-Domain)", OpticalSetupTreeElement.StratifiedMediaComponent(StratifiedMediaComponentSubType.PlaneSurface));
			lensAnalysisCollection.Add("Components|Plane Surface \\w Coating (k-Domain)", OpticalSetupTreeElement.StratifiedMediaComponent(StratifiedMediaComponentSubType.PlaneSurface));
			lensAnalysisCollection.Add("Components|Curved Surface (x-Domain)", OpticalSetupTreeElement.CurvedSurface());
			lensAnalysisCollection.Add("Components|Off-Axis Parabolic Mirror (x-Domain)", OpticalSetupTreeElement.OffAxisParabolicMirror());
			lensAnalysisCollection.Add("Components|Spherical Lens (x-Domain)", OpticalSetupTreeElement.SphericalLens());
			lensAnalysisCollection.Add("Components|Lens System (x-Domain)", OpticalSetupTreeElement.LensSystem());
			#endregion

			#region ideal components
			lensAnalysisCollection.Add("Functions|Aperture", OpticalSetupTreeElement.Aperture());
			lensAnalysisCollection.Add("Functions|Stop", OpticalSetupTreeElement.Stop());
			lensAnalysisCollection.Add("Functions|Ideal Lens (Lens Equation)", OpticalSetupTreeElement.FromComponentCatalog("Ideal Lens (Lens Equation)"));
			lensAnalysisCollection.Add("Functions|Ideal Lens (Collimation Mode)", OpticalSetupTreeElement.FromComponentCatalog("Idealized Lens [Collimation Mode]"));
			lensAnalysisCollection.Add("Functions|Ideal Lens (Focusing Mode)", OpticalSetupTreeElement.FromComponentCatalog("Idealized Lens [Focusing Mode]"));
			lensAnalysisCollection.Add("Functions|Zernike & Seidel Aberrations", OpticalSetupTreeElement.ZernikeAndSeidelAberrations());
			#endregion

			#region detectors
			lensAnalysisCollection.Add("Universal Detector", OpticalSetupTreeElement.UniversalDetector());
			lensAnalysisCollection.Add("Detectors|Detector from Catalog", OpticalSetupTreeElement.LoadFromDetectorCatalog());
			lensAnalysisCollection.Add("Detectors|Irradiance", OpticalSetupTreeElement.FromDetectorCatalog("Irradiance"));
			lensAnalysisCollection.Add("Detectors|Spot Size", OpticalSetupTreeElement.FromDetectorCatalog("Beam/Spot Size (Field)"));
			lensAnalysisCollection.Add("Detectors|Wavefront Error", OpticalSetupTreeElement.WavefrontError());
			lensAnalysisCollection.Add("Detectors|Zernike & Seidel Aberrations", OpticalSetupTreeElement.ZernikeAndSeidelAberrationsDetector());
			#endregion

			return new OpticalSetupTree("OS Tree for Use Case", lensAnalysisCollection);
		}

		/// <summary>
		/// Main method only executed when you start this module by your own for test purposes.
		/// </summary>
		public void Run() {
			// Refresh the available trees in all open Optical Setups
			OpticalSetupTreeSupport.ReloadFromHardDisc();
		}
	}
}
